@@ -51,6 +51,7 @@ sub login {
     # (double cookies, odd compat code settings, etc)
     return $user if $user->id;
 
+    $type = LOGIN_REQUIRED if Bugzilla->cgi->param('GoAheadAndLogIn');
     $type = LOGIN_NORMAL unless defined $type;
 
     # Log in using whatever methods are defined in user_info_class.
@@ -70,10 +71,11 @@ sub login {
     if ($userid) {
         $user = new Bugzilla::User($userid);
 
+        # Redirect to SSL if required
+        if (Param('sslbase') ne '' and Param('ssl') ne 'never') {
+            Bugzilla->cgi->require_https(Param('sslbase'));
+        }
         $user->set_flags('can_logout' => $class->can_logout);
-
-        # Compat stuff
-        $::userid = $userid;
     } else {
         Bugzilla->logout_request();
     }
@@ -98,10 +100,12 @@ Bugzilla::Auth::Login::WWW - WWW login information gathering module
 
 =head1 METHODS
 
+=over
+
 =item C<login>
 
 Passes C<login> calls to each class defined in the param C<user_info_class>
 and returns a C<Bugzilla::User> object from the first one that successfully
 gathers user login information.
 
-
+=back
