@@ -35,6 +35,7 @@ use Bugzilla::Config;
 use Bugzilla::Constants;
 use Bugzilla::Error;
 use Bugzilla::Util;
+use Bugzilla::Token;
 
 sub login {
     my ($class, $type) = @_;
@@ -45,13 +46,16 @@ sub login {
     }
 
     my $cgi = Bugzilla->cgi;
+    my $dbh = Bugzilla->dbh;
 
     # First, try the actual login method against form variables
     my $username = $cgi->param("Bugzilla_login");
     my $passwd = $cgi->param("Bugzilla_password");
+    
+    $cgi->delete('Bugzilla_login', 'Bugzilla_password');
 
-    my $authmethod = Param("user_verify_class");
-    my ($authres, $userid, $extra, $info) =
+    # Perform the actual authentication, get the method name from the class name
+    my ($authmethod, $authres, $userid, $extra, $info) =
       Bugzilla::Auth->authenticate($username, $passwd);
 
     if ($authres == AUTH_OK) {
@@ -67,17 +71,18 @@ sub login {
         # subsequent login
         trick_taint($ipaddr);
 
-        my $dbh = Bugzilla->dbh;
-        $dbh->do("INSERT INTO logincookies (userid, ipaddr, lastused)
-                 VALUES (?, ?, NOW())",
+        my $logincookie = Bugzilla::Token::GenerateUniqueToken('logincookies', 'cookie');
+
+        $dbh->do("INSERT INTO logincookies (cookie, userid, ipaddr, lastused)
+                 VALUES (?, ?, ?, NOW())",
                  undef,
-                 $userid, $ipaddr);
-        my $logincookie = $dbh->selectrow_array("SELECT LAST_INSERT_ID()");
+                 $logincookie, $userid, $ipaddr);
 
         # Remember cookie only if admin has told so
         # or admin didn't forbid it and user told to remember.
         if ((Param('rememberlogin') eq 'on') ||
             ((Param('rememberlogin') ne 'off') &&
+             $cgi->param('Bugzilla_remember') &&
              ($cgi->param('Bugzilla_remember') eq 'on'))) {
             $cgi->send_cookie(-name => 'Bugzilla_login',
                               -value => $userid,
@@ -135,6 +140,12 @@ sub login {
     # No login details were given, but we require a login if the
     # page does
     if ($authres == AUTH_NODATA && $type == LOGIN_REQUIRED) {
+
+        # Redirect to SSL if required
+        if (Param('sslbase') ne '' and Param('ssl') ne 'never') {
+            $cgi->require_https(Param('sslbase'));
+        }
+    
         # Throw up the login page
 
         print Bugzilla->cgi->header();
@@ -142,8 +153,6 @@ sub login {
         my $template = Bugzilla->template;
         $template->process("account/auth/login.html.tmpl",
                            { 'target' => $cgi->url(-relative=>1),
-                             'form' => \%::FORM,
-                             'mform' => \%::MFORM,
                              'caneditaccount' => Bugzilla::Auth->can_edit('new'),
                              'has_db' => Bugzilla::Auth->has_db,
                            }
@@ -153,8 +162,9 @@ sub login {
         # This seems like as good as time as any to get rid of old
         # crufty junk in the logincookies table.  Get rid of any entry
         # that hasn't been used in a month.
-        Bugzilla->dbh->do("DELETE FROM logincookies " .
-                          "WHERE TO_DAYS(NOW()) - TO_DAYS(lastused) > 30");
+        $dbh->do("DELETE FROM logincookies WHERE " .
+                 $dbh->sql_to_days('NOW()') . " - " .
+                 $dbh->sql_to_days('lastused') . " > 30");
 
         exit;
     }
@@ -177,7 +187,7 @@ sub login {
 
     # If we get here, then we've run out of options, which shouldn't happen
     ThrowCodeError("authres_unhandled", { authres => $authres, 
-                                          type => $type, });
+                                          type => $type });
 }
 
 # This auth style allows the user to log out.
@@ -188,6 +198,7 @@ sub can_logout { return 1; }
 sub logout {
     my ($class, $user, $option) = @_;
     my $dbh = Bugzilla->dbh;
+    my $cgi = Bugzilla->cgi;
     $option = LOGOUT_ALL unless defined $option;
 
     if ($option == LOGOUT_ALL) {
@@ -196,9 +207,18 @@ sub logout {
             return;
     }
 
-    # The LOGOUT_*_CURRENT options require a cookie 
-    my $cookie = Bugzilla->cgi->cookie("Bugzilla_logincookie");
-    detaint_natural($cookie);
+    # The LOGOUT_*_CURRENT options require the current login cookie.
+    # If a new cookie has been issued during this run, that's the current one.
+    # If not, it's the one we've received.
+    my $cookie;
+    foreach (@{$cgi->{'Bugzilla_cookie_list'}}) {
+        if ($_->name() eq 'Bugzilla_logincookie') {
+            $cookie = $_->value();
+            last;
+        }
+    }
+    $cookie ||= $cgi->cookie("Bugzilla_logincookie");
+    trick_taint($cookie);
 
     # These queries use both the cookie ID and the user ID as keys. Even
     # though we know the userid must match, we still check it in the SQL
@@ -225,12 +245,8 @@ sub logout {
 
 sub clear_browser_cookies {
     my $cgi = Bugzilla->cgi;
-    $cgi->send_cookie(-name => "Bugzilla_login",
-                      -value => "",
-                      -expires => "Tue, 15-Sep-1998 21:49:00 GMT");
-    $cgi->send_cookie(-name => "Bugzilla_logincookie",
-                      -value => "",
-                      -expires => "Tue, 15-Sep-1998 21:49:00 GMT");
+    $cgi->remove_cookie('Bugzilla_login');
+    $cgi->remove_cookie('Bugzilla_logincookie');
 }
 
 1;
