@@ -25,6 +25,14 @@ use strict;
 
 package Bugzilla::CGI;
 
+BEGIN {
+    if ($^O =~ /MSWin32/i) {
+        # Help CGI find the correct temp directory as the default list
+        # isn't Windows friendly (Bug 248988)
+        $ENV{'TMPDIR'} = $ENV{'TEMP'} || $ENV{'TMP'} || "$ENV{'WINDIR'}\\TEMP";
+    }
+}
+
 use CGI qw(-no_xhtml -oldstyle_urls :private_tempfiles :unique_headers SERVER_PUSH);
 
 use base qw(CGI);
@@ -48,11 +56,21 @@ sub new {
 
     my $self = $class->SUPER::new(@args);
 
+    # This happens here so that command-line scripts don't spit out
+    # their errors in HTML format.
+    require CGI::Carp;
+    import CGI::Carp qw(fatalsToBrowser);
+
     # Make sure our outgoing cookie list is empty on each invocation
     $self->{Bugzilla_cookie_list} = [];
 
-    # Make sure that we don't send any charset headers
-    $self->charset('');
+    # Send appropriate charset
+    $self->charset(Param('utf8') ? 'UTF-8' : '');
+
+    # Redirect to SSL if required
+    if (Param('sslbase') ne '' and Param('ssl') eq 'always' and i_am_cgi()) {
+        $self->require_https(Param('sslbase'));
+    }
 
     # Check for errors
     # All of the Bugzilla code wants to do this, so do it here instead of
@@ -102,7 +120,7 @@ sub canonicalise_query {
         my $esc_key = url_quote($key);
 
         foreach my $value ($self->param($key)) {
-            if ($value) {
+            if (defined($value)) {
                 my $esc_value = url_quote($value);
 
                 push(@parameters, "$esc_key=$esc_value");
@@ -134,7 +152,7 @@ sub multipart_init {
     # Note: CGI.pm::multipart_init up to v3.04 explicitly set nph to 0
     # CGI.pm::multipart_init v3.05 explicitly sets nph to 1
     # CGI.pm's header() sets nph according to a param or $CGI::NPH, which
-    # is the desired behavour.
+    # is the desired behaviour.
 
     # Allow multiple calls to $cgi->header()
     $CGI::HEADERS_ONCE = 0;
@@ -169,7 +187,7 @@ sub multipart_start {
 }
 
 # The various parts of Bugzilla which create cookies don't want to have to
-# pass them arround to all of the callers. Instead, store them locally here,
+# pass them around to all of the callers. Instead, store them locally here,
 # and then output as required from |header|.
 sub send_cookie {
     my $self = shift;
@@ -188,8 +206,9 @@ sub send_cookie {
         ThrowCodeError('cookies_need_value');
     }
 
-    # Add the default path in.
+    # Add the default path and the domain in.
     $paramhash{'-path'} = Param('cookiepath');
+    $paramhash{'-domain'} = Param('cookiedomain') if Param('cookiedomain');
 
     # Move the param list back into an array for the call to cookie().
     foreach (keys(%paramhash)) {
@@ -209,7 +228,22 @@ sub remove_cookie {
     $self->send_cookie('-name'    => $cookiename,
                        '-expires' => 'Tue, 15-Sep-1998 21:49:00 GMT',
                        '-value'   => 'X');
+}
 
+# Redirect to https if required
+sub require_https {
+    my $self = shift;
+    if ($self->protocol ne 'https') {
+        my $url = shift;
+        if (defined $url) {
+            $url .= $self->url('-path_info' => 1, '-query' => 1, '-relative' => 1);
+        } else {
+            $url = $self->self_url;
+            $url =~ s/^http:/https:/i;
+        }
+        print $self->redirect(-location => $url);
+        exit;
+    }
 }
 
 1;
@@ -274,6 +308,14 @@ effectively removing the cookie.
 
 As its only argument, it takes the name of the cookie to expire.
 
+=item C<require_https($baseurl)>
+
+This routine checks if the current page is being served over https, and
+redirects to the https protocol if required, retaining QUERY_STRING.
+
+It takes an option argument which will be used as the base URL.  If $baseurl
+is not provided, the current URL is used.
+
 =back
 
 =head1 SEE ALSO
