@@ -18,6 +18,8 @@
 # Rights Reserved.
 #
 # Contributor(s): Gervase Markham <gerv@gerv.net>
+#                 Albert Ting <altlst@sonic.net>
+#                 A. Karl Kornel <karl@kornel.name>
 
 use strict;
 use lib ".";
@@ -33,6 +35,8 @@ package Bugzilla::Chart;
 use Bugzilla::Util;
 use Bugzilla::Series;
 
+use Date::Format;
+
 sub new {
     my $invocant = shift;
     my $class = ref($invocant) || $invocant;
@@ -115,16 +119,11 @@ sub add {
     my $self = shift;
     my @series_ids = @_;
 
-    # If we are going from < 2 to >= 2 series, add the Grand Total line.
-    if (!$self->{'gt'}) {
-        my $current_size = scalar($self->getSeriesIDs());
-        if ($current_size < 2 &&
-            $current_size + scalar(@series_ids) >= 2) 
-        {
-            $self->{'gt'} = 1;
-        }
-    }
-        
+    # Get the current size of the series; required for adding Grand Total later
+    my $current_size = scalar($self->getSeriesIDs());
+    
+    # Count the number of added series
+    my $added = 0;
     # Create new Series and push them on to the list of lines.
     # Note that new lines have no label; the display template is responsible
     # for inventing something sensible.
@@ -133,6 +132,16 @@ sub add {
         if ($series) {
             push(@{$self->{'lines'}}, [$series]);
             push(@{$self->{'labels'}}, "");
+            $added++;
+        }
+    }
+    
+    # If we are going from < 2 to >= 2 series, add the Grand Total line.
+    if (!$self->{'gt'}) {
+        if ($current_size < 2 &&
+            $current_size + $added >= 2) 
+        {
+            $self->{'gt'} = 1;
         }
     }
 }
@@ -200,6 +209,7 @@ sub data {
 sub readData {
     my $self = shift;
     my @data;
+    my @maxvals;
 
     # Note: you get a bad image if getSeriesIDs returns nothing
     # We need to handle errors better.
@@ -228,14 +238,18 @@ sub readData {
         $dateto = $self->{'dateto'};
     }
 
+    # Convert UNIX times back to a date format usable for SQL queries.
+    my $sql_from = time2str('%Y-%m-%d', $datefrom);
+    my $sql_to = time2str('%Y-%m-%d', $dateto);
+
     # Prepare the query which retrieves the data for each series
-    my $query = "SELECT TO_DAYS(series_date) - " . 
-                "  TO_DAYS(FROM_UNIXTIME($datefrom)), " . 
-                "series_value FROM series_data " .
+    my $query = "SELECT " . $dbh->sql_to_days('series_date') . " - " .
+                            $dbh->sql_to_days('?') . ", series_value " .
+                "FROM series_data " .
                 "WHERE series_id = ? " .
-                "AND series_date >= FROM_UNIXTIME($datefrom)";
+                "AND series_date >= ?";
     if ($dateto) {
-        $query .= " AND series_date <= FROM_UNIXTIME($dateto)";
+        $query .= " AND series_date <= ?";
     }
     
     my $sth = $dbh->prepare($query);
@@ -243,33 +257,94 @@ sub readData {
     my $gt_index = $self->{'gt'} ? scalar(@{$self->{'lines'}}) : undef;
     my $line_index = 0;
 
+    $maxvals[$gt_index] = 0 if $gt_index;
+
+    my @datediff_total;
+
     foreach my $line (@{$self->{'lines'}}) {        
         # Even if we end up with no data, we need an empty arrayref to prevent
         # errors in the PNG-generating code
         $data[$line_index] = [];
+        $maxvals[$line_index] = 0;
 
         foreach my $series (@$line) {
 
             # Get the data for this series and add it on
-            $sth->execute($series->{'series_id'});
+            if ($dateto) {
+                $sth->execute($sql_from, $series->{'series_id'}, $sql_from, $sql_to);
+            }
+            else {
+                $sth->execute($sql_from, $series->{'series_id'}, $sql_from);
+            }
             my $points = $sth->fetchall_arrayref();
 
             foreach my $point (@$points) {
                 my ($datediff, $value) = @$point;
                 $data[$line_index][$datediff] ||= 0;
                 $data[$line_index][$datediff] += $value;
+                if ($data[$line_index][$datediff] > $maxvals[$line_index]) {
+                    $maxvals[$line_index] = $data[$line_index][$datediff];
+                }
+
+                $datediff_total[$datediff] += $value;
 
                 # Add to the grand total, if we are doing that
                 if ($gt_index) {
                     $data[$gt_index][$datediff] ||= 0;
                     $data[$gt_index][$datediff] += $value;
+                    if ($data[$gt_index][$datediff] > $maxvals[$gt_index]) {
+                        $maxvals[$gt_index] = $data[$gt_index][$datediff];
+                    }
                 }
             }
         }
 
+        # We are done with the series making up this line, go to the next one
         $line_index++;
     }
 
+    # calculate maximum y value
+    if ($self->{'cumulate'}) {
+        # Make sure we do not try to take the max of an array with undef values
+        my @processed_datediff;
+        while (@datediff_total) {
+            my $datediff = shift @datediff_total;
+            push @processed_datediff, $datediff if defined($datediff);
+        }
+        $self->{'y_max_value'} = Bugzilla::Util::max(@processed_datediff);
+    }
+    else {
+        $self->{'y_max_value'} = Bugzilla::Util::max(@maxvals);
+    }
+    $self->{'y_max_value'} |= 1; # For log()
+
+    # Align the max y value:
+    #  For one- or two-digit numbers, increase y_max_value until divisible by 8
+    #  For larger numbers, see the comments below to figure out what's going on
+    if ($self->{'y_max_value'} < 100) {
+        do {
+            ++$self->{'y_max_value'};
+        } while ($self->{'y_max_value'} % 8 != 0);
+    }
+    else {
+        #  First, get the # of digits in the y_max_value
+        my $num_digits = 1+int(log($self->{'y_max_value'})/log(10));
+
+        # We want to zero out all but the top 2 digits
+        my $mask_length = $num_digits - 2;
+        $self->{'y_max_value'} /= 10**$mask_length;
+        $self->{'y_max_value'} = int($self->{'y_max_value'});
+        $self->{'y_max_value'} *= 10**$mask_length;
+
+        # Add 10^$mask_length to the max value
+        # Continue to increase until it's divisible by 8 * 10^($mask_length-1)
+        # (Throwing in the -1 keeps at least the smallest digit at zero)
+        do {
+            $self->{'y_max_value'} += 10**$mask_length;
+        } while ($self->{'y_max_value'} % (8*(10**($mask_length-1))) != 0);
+    }
+
+        
     # Add the x-axis labels into the data structure
     my $date_progression = generateDateProgression($datefrom, $dateto);
     unshift(@data, $date_progression);
@@ -320,7 +395,8 @@ sub getVisibleSeries {
                         "    AND cgm.group_id NOT IN($grouplist) " .
                         "WHERE creator = " . Bugzilla->user->id . " OR " .
                         "      cgm.category_id IS NULL " . 
-                        "GROUP BY series_id");
+                   $dbh->sql_group_by('series.series_id', 'cc1.name, cc2.name, ' .
+                                      'series.name'));
     foreach my $series (@$serieses) {
         my ($cat, $subcat, $name, $series_id) = @$series;
         $cats{$cat}{$subcat}{$name} = $series_id;
