@@ -26,6 +26,7 @@
 #                 Bradley Baetz <bbaetz@student.usyd.edu.au>
 #                 Christopher Aillon <christopher@aillon.com>
 #                 Shane H. W. Travis <travis@sedsystems.ca>
+#                 Max Kanat-Alexander <mkanat@bugzilla.org>
 
 package Bugzilla::Constants;
 use strict;
@@ -51,16 +52,47 @@ use base qw(Exporter);
     LOGOUT_CURRENT
     LOGOUT_KEEP_CURRENT
 
-    DEFAULT_FLAG_EMAIL_SETTINGS
-    DEFAULT_EMAIL_SETTINGS
-
     GRANT_DIRECT
-    GRANT_DERIVED
     GRANT_REGEXP
 
     GROUP_MEMBERSHIP
     GROUP_BLESS
     GROUP_VISIBLE
+
+    MAILTO_USER
+    MAILTO_GROUP
+
+    DEFAULT_COLUMN_LIST
+    DEFAULT_QUERY_NAME
+
+    QUERY_LIST
+    LIST_OF_BUGS
+
+    COMMENT_COLS
+
+    UNLOCK_ABORT
+    
+    RELATIONSHIPS
+    REL_ASSIGNEE REL_QA REL_REPORTER REL_CC REL_VOTER 
+    REL_ANY
+    
+    POS_EVENTS
+    EVT_OTHER EVT_ADDED_REMOVED EVT_COMMENT EVT_ATTACHMENT EVT_ATTACHMENT_DATA
+    EVT_PROJ_MANAGEMENT EVT_OPENED_CLOSED EVT_KEYWORD EVT_CC EVT_DEPEND_BLOCK
+    
+    NEG_EVENTS
+    EVT_UNCONFIRMED EVT_CHANGED_BY_ME 
+        
+    GLOBAL_EVENTS
+    EVT_FLAG_REQUESTED EVT_REQUESTED_FLAG
+
+    FULLTEXT_BUGLIST_LIMIT
+
+    ADMIN_GROUP_NAME
+
+    SENDMAIL_EXE
+
+    SAFE_PROTOCOLS
 );
 
 @Bugzilla::Constants::EXPORT_OK = qw(contenttypes);
@@ -116,87 +148,106 @@ use constant LOGOUT_KEEP_CURRENT => 2;
 
 use constant contenttypes =>
   {
-   "html" => "text/html" , 
+   "html"=> "text/html" , 
    "rdf" => "application/rdf+xml" , 
-   "xml" => "text/xml" , 
-   "js" => "application/x-javascript" , 
+   "atom"=> "application/atom+xml" ,
+   "xml" => "application/xml" , 
+   "js"  => "application/x-javascript" , 
    "csv" => "text/plain" ,
    "png" => "image/png" ,
    "ics" => "text/calendar" ,
   };
 
-use constant DEFAULT_FLAG_EMAIL_SETTINGS =>
-      "~FlagRequestee~on" .
-      "~FlagRequester~on";
-
-# By default, almost all bugmail is turned on, with the exception
-# of CC list additions for anyone except the Assignee/Owner.
-# If you want to customize the default settings for new users at
-# your own site, ensure that each of the lines ends with either
-# "~on" or just "~" (for off).
-
-use constant DEFAULT_EMAIL_SETTINGS => 
-      "ExcludeSelf~on" .
-
-      "~FlagRequestee~on" .
-      "~FlagRequester~on" .
-
-      "~emailOwnerRemoveme~on" .
-      "~emailOwnerComments~on" .
-      "~emailOwnerAttachments~on" .
-      "~emailOwnerStatus~on" .
-      "~emailOwnerResolved~on" .
-      "~emailOwnerKeywords~on" .
-      "~emailOwnerCC~on" .
-      "~emailOwnerOther~on" .
-      "~emailOwnerUnconfirmed~on" .
-  
-      "~emailReporterRemoveme~on" .
-      "~emailReporterComments~on" .
-      "~emailReporterAttachments~on" .
-      "~emailReporterStatus~on" .
-      "~emailReporterResolved~on" .
-      "~emailReporterKeywords~on" .
-      "~emailReporterCC~" .
-      "~emailReporterOther~on" .
-      "~emailReporterUnconfirmed~on" .
-  
-      "~emailQAcontactRemoveme~on" .
-      "~emailQAcontactComments~on" .
-      "~emailQAcontactAttachments~on" .
-      "~emailQAcontactStatus~on" .
-      "~emailQAcontactResolved~on" .
-      "~emailQAcontactKeywords~on" .
-      "~emailQAcontactCC~" .
-      "~emailQAcontactOther~on" .
-      "~emailQAcontactUnconfirmed~on" .
-  
-      "~emailCClistRemoveme~on" .
-      "~emailCClistComments~on" .
-      "~emailCClistAttachments~on" .
-      "~emailCClistStatus~on" .
-      "~emailCClistResolved~on" .
-      "~emailCClistKeywords~on" .
-      "~emailCClistCC~" .
-      "~emailCClistOther~on" .
-      "~emailCClistUnconfirmed~on" .
-  
-      "~emailVoterRemoveme~on" .
-      "~emailVoterComments~on" .
-      "~emailVoterAttachments~on" .
-      "~emailVoterStatus~on" .
-      "~emailVoterResolved~on" .
-      "~emailVoterKeywords~on" .
-      "~emailVoterCC~" .
-      "~emailVoterOther~on" .
-      "~emailVoterUnconfirmed~on";
-
 use constant GRANT_DIRECT => 0;
-use constant GRANT_DERIVED => 1;
 use constant GRANT_REGEXP => 2;
 
 use constant GROUP_MEMBERSHIP => 0;
 use constant GROUP_BLESS => 1;
 use constant GROUP_VISIBLE => 2;
 
+use constant MAILTO_USER => 0;
+use constant MAILTO_GROUP => 1;
+
+# The default list of columns for buglist.cgi
+use constant DEFAULT_COLUMN_LIST => (
+    "bug_severity", "priority", "op_sys","assigned_to",
+    "bug_status", "resolution", "short_short_desc"
+);
+
+# Used by query.cgi and buglist.cgi as the named-query name
+# for the default settings.
+use constant DEFAULT_QUERY_NAME => '(Default query)';
+
+# The possible types for saved searches.
+use constant QUERY_LIST => 0;
+use constant LIST_OF_BUGS => 1;
+
+# The column length for displayed (and wrapped) bug comments.
+use constant COMMENT_COLS => 80;
+
+# used by Bugzilla::DB to indicate that tables are being unlocked
+# because of error
+use constant UNLOCK_ABORT => 1;
+
+use constant REL_ASSIGNEE           => 0;
+use constant REL_QA                 => 1;
+use constant REL_REPORTER           => 2;
+use constant REL_CC                 => 3;
+use constant REL_VOTER              => 4;
+
+use constant RELATIONSHIPS => REL_ASSIGNEE, REL_QA, REL_REPORTER, REL_CC, 
+                              REL_VOTER;
+                              
+# Used for global events like EVT_FLAG_REQUESTED
+use constant REL_ANY                => 100;
+
+# There are two sorts of event - positive and negative. Positive events are
+# those for which the user says "I want mail if this happens." Negative events
+# are those for which the user says "I don't want mail if this happens."
+#
+# Exactly when each event fires is defined in wants_bug_mail() in User.pm; I'm
+# not commenting them here in case the comments and the code get out of sync.
+use constant EVT_OTHER              => 0;
+use constant EVT_ADDED_REMOVED      => 1;
+use constant EVT_COMMENT            => 2;
+use constant EVT_ATTACHMENT         => 3;
+use constant EVT_ATTACHMENT_DATA    => 4;
+use constant EVT_PROJ_MANAGEMENT    => 5;
+use constant EVT_OPENED_CLOSED      => 6;
+use constant EVT_KEYWORD            => 7;
+use constant EVT_CC                 => 8;
+use constant EVT_DEPEND_BLOCK       => 9;
+
+use constant POS_EVENTS => EVT_OTHER, EVT_ADDED_REMOVED, EVT_COMMENT, 
+                           EVT_ATTACHMENT, EVT_ATTACHMENT_DATA, 
+                           EVT_PROJ_MANAGEMENT, EVT_OPENED_CLOSED, EVT_KEYWORD,
+                           EVT_CC, EVT_DEPEND_BLOCK;
+
+use constant EVT_UNCONFIRMED        => 50;
+use constant EVT_CHANGED_BY_ME      => 51;
+
+use constant NEG_EVENTS => EVT_UNCONFIRMED, EVT_CHANGED_BY_ME;
+
+# These are the "global" flags, which aren't tied to a particular relationship.
+# and so use REL_ANY.
+use constant EVT_FLAG_REQUESTED     => 100; # Flag has been requested of me
+use constant EVT_REQUESTED_FLAG     => 101; # I have requested a flag
+
+use constant GLOBAL_EVENTS => EVT_FLAG_REQUESTED, EVT_REQUESTED_FLAG;
+
+#  Number of bugs to return in a buglist when performing
+#  a fulltext search.
+use constant FULLTEXT_BUGLIST_LIMIT => 200;
+
+# Default administration group name.
+use constant ADMIN_GROUP_NAME => 'admin';
+
+# Path to sendmail.exe (Windows only)
+use constant SENDMAIL_EXE => '/usr/lib/sendmail.exe';
+
+# Protocols which are considered as safe.
+use constant SAFE_PROTOCOLS => ('afs', 'cid', 'ftp', 'gopher', 'http', 'https',
+                                'irc', 'mid', 'news', 'nntp', 'prospero', 'telnet',
+                                'view-source', 'wais');
+
 1;
