@@ -87,12 +87,7 @@
 ],
 
 'search/search-specific.html.tmpl' => [
-  's',
-],
-
-'search/tabs.html.tmpl' => [
-  'tab.name',
-  'tab.description',
+  'status.name',
 ],
 
 'request/queue.html.tmpl' => [
@@ -105,14 +100,9 @@
 
 'reports/components.html.tmpl' => [
   'numcols',
-  'comp.description', 
 ],
 
 'reports/duplicates-table.html.tmpl' => [
-  '"&amp;maxrows=$maxrows" IF maxrows',
-  '"&amp;changedsince=$changedsince" IF changedsince',
-  '"&amp;bug_id=$bug_ids_string&amp;sortvisible=1" 
-                                                            IF sortvisible',
   'column.name', 
   'column.description',
   'bug.id', 
@@ -128,14 +118,12 @@
 ],
 
 'reports/keywords.html.tmpl' => [
-  'keyword.description', 
   'keyword.bugcount', 
 ],
 
 'reports/report-table.csv.tmpl' => [
-  'num_bugs',
   'data.$tbl.$col.$row',
-  'title',
+  'colsepchar',
 ],
 
 'reports/report-table.html.tmpl' => [
@@ -190,6 +178,7 @@
 
 'reports/chart.csv.tmpl' => [
   'data.$j.$i', 
+  'colsepchar',
 ],
 
 'reports/create-chart.html.tmpl' => [
@@ -207,20 +196,10 @@
 
 'list/edit-multiple.html.tmpl' => [
   'group.id', 
-  'group.description',
-  'group.description FILTER inactive', 
   'knum', 
   'menuname', 
 ],
 
-'list/list-simple.html.tmpl' => [
-  'title', 
-],
-
-'list/list.html.tmpl' => [
-  'buglist', 
-],
-
 'list/list.rdf.tmpl' => [
   'template_version', 
   'bug.bug_id', 
@@ -235,6 +214,7 @@
 
 'list/list.csv.tmpl' => [
   'bug.bug_id', 
+  'colsepchar',
 ],
 
 'list/list.js.tmpl' => [
@@ -250,13 +230,8 @@
   'VERSION', 
 ],
 
-'global/choose-classification.html.tmpl' => [
-  'classdesc.$p', 
-],
-
 'global/choose-product.html.tmpl' => [
   'target',
-  'proddesc.$p', 
 ],
 
 # You are not permitted to add any values here. Everything in this file should 
@@ -268,9 +243,8 @@
 'global/header.html.tmpl' => [
   'javascript', 
   'style', 
-  'bgcolor', 
   'onload',
-  'bodyattrs', 
+  'title',
   'h1',
   'h2',
   'h3', 
@@ -313,7 +287,8 @@
 
 'bug/comments.html.tmpl' => [
   'comment.isprivate', 
-  'comment.when', 
+  'comment.time', 
+  'bug.bug_id',
 ],
 
 'bug/dependency-graph.html.tmpl' => [
@@ -337,19 +312,17 @@
 ],
 
 'bug/edit.html.tmpl' => [
+  'bug.deadline',
   'bug.remaining_time', 
   'bug.delta_ts', 
   'bug.bug_id', 
   'bug.votes', 
   'group.bit', 
-  'group.description', 
   'dep.title', 
   'dep.fieldname', 
   'accesskey', 
   'bug.${dep.fieldname}.join(\', \')', 
   'selname',
-  'depbug FILTER bug_link(depbug)',
-  '"${bug.dup_id}" FILTER bug_link(bug.dup_id)',
 ],
 
 'bug/knob.html.tmpl' => [
@@ -366,14 +339,27 @@
 
 'bug/show-multiple.html.tmpl' => [
   'bug.bug_id', 
+  'bug.deadline',
+],
+
+'bug/show.html.tmpl' => [
+  'bug.bug_id',
 ],
 
 'bug/show.xml.tmpl' => [
   'VERSION', 
-  'a.attachid', 
+  'a.id', 
   'field', 
 ],
 
+'bug/summarize-time.html.tmpl' => [
+  'global.grand_total FILTER format("%.2f")',
+  'subtotal FILTER format("%.2f")',
+  'work_time FILTER format("%.2f")',
+  'global.total FILTER format("%.2f")',
+],
+
+
 'bug/time.html.tmpl' => [
   'time_unit FILTER format(\'%.1f\')', 
   'time_unit FILTER format(\'%.2f\')', 
@@ -382,7 +368,7 @@
 ],
 
 'bug/votes/list-for-bug.html.tmpl' => [
-  'voter.count', 
+  'voter.vote_count', 
   'total', 
 ],
 
@@ -403,24 +389,17 @@
   'bug_id', 
 ],
 
-'bug/process/next.html.tmpl' => [
-  'bug.bug_id', 
-],
-
 'bug/process/results.html.tmpl' => [
   'title.$type', 
   'id', 
-],
-
-'bug/create/comment.txt.tmpl' => [
-  'form.comment', 
+  'linktext.$type',
 ],
 
 'bug/create/create.html.tmpl' => [
   'g.bit',
-  'g.description',
   'sel.name',
   'sel.description',
+  'cloned_bug_id'
 ],
 
 'bug/create/create-guided.html.tmpl' => [
@@ -452,19 +431,19 @@
 ],
 
 'attachment/edit.html.tmpl' => [
-  'attachid', 
-  'bugid', 
+  'attachment.id', 
+  'attachment.bug_id', 
   'a', 
 ],
 
 'attachment/list.html.tmpl' => [
-  'attachment.attachid', 
+  'attachment.id', 
   'flag.status',
   'bugid',
 ],
 
 'attachment/show-multiple.html.tmpl' => [
-  'a.attachid', 
+  'a.id',
   'flag.status'
 ],
 
@@ -494,24 +473,7 @@
 ],
 
 'admin/table.html.tmpl' => [
-  'link_uri',
-  'c.content'
-],
-
-'admin/classifications/del.html.tmpl' => [
-  'description', 
-],
-
-'admin/classifications/edit.html.tmpl' => [
-  'description', 
-],
-
-'admin/classifications/reclassify.html.tmpl' => [
-  'description', 
-],
-
-'admin/classifications/select.html.tmpl' => [
-  'cl.description', 
+  'link_uri'
 ],
 
 'admin/products/groupcontrol/confirm-edit.html.tmpl' => [
@@ -519,8 +481,6 @@
 ],
 
 'admin/products/groupcontrol/edit.html.tmpl' => [
-  'filt_classification', 
-  'filt_product', 
   'group.bugcount', 
   'group.id', 
   'const.CONTROLMAPNA', 
@@ -529,6 +489,19 @@
   'const.CONTROLMAPMANDATORY', 
 ],
 
+'admin/products/list.html.tmpl' => [
+  'classification_url_part', 
+],
+
+'admin/products/confirm-delete.html.tmpl' => [
+  'classification_url_part', 
+],
+
+'admin/products/footer.html.tmpl' => [
+  'classification_url_part', 
+  'classification_text', 
+],
+
 'admin/keywords/edit.html.tmpl' => [
   'keyword_id',
   'bug_count',
@@ -551,32 +524,50 @@
   'type.sortkey || 1',
   'typeLabelLowerPlural',
   'typeLabelLowerSingular',
+  'selname',
 ],
 
 'admin/flag-type/list.html.tmpl' => [
   'type.id', 
-  'type.flag_count', 
 ],
 
 
 'admin/components/confirm-delete.html.tmpl' => [
-  'bug_count'
+  'comp.bug_count'
 ],
 
 'admin/components/deleted.html.tmpl' => [
   'deleted_bug_count'
 ],
 
+'admin/users/confirm-delete.html.tmpl' => [
+  'andstring',
+  'responsibilityterms.$responsibility',
+  'reporter',
+  'assignee_or_qa',
+  'cc',
+  'flags.requestee',
+  'flags.setter',
+  'longdescs',
+  'namedqueries',
+  'votes',
+  'series',
+  'watch.watched',
+  'watch.watcher',
+  'whine_events',
+  'whine_schedules',
+  'otheruser.id'
+],
+
+'admin/users/edit.html.tmpl' => [
+  'otheruser.id',
+  'group.id',
+  'perms.directbless',
+  'perms.directmember',
+],
+
 'admin/components/edit.html.tmpl' => [
-  'bug_count'
-],
-
-'admin/components/list.html.tmpl' => [
-  'cgi.query_string'
-],
-
-'admin/components/select-product.html.tmpl' => [
-  'cgi.query_string'
+  'comp.bug_count'
 ],
 
 'admin/milestones/confirm-delete.html.tmpl' => [
@@ -584,7 +575,11 @@
 ],
 
 'admin/milestones/deleted.html.tmpl' => [
-  'deleted_bug_count'
+  'bug_count'
+],
+
+'admin/versions/confirm-delete.html.tmpl' => [
+  'bug_count'
 ],
 
 'account/login.html.tmpl' => [
@@ -592,21 +587,14 @@
 ],
 
 'account/prefs/email.html.tmpl' => [
-  'role', 
-  'reason.name', 
-  'reason.description',
-],
-
-'account/prefs/permissions.html.tmpl' => [
-  'bit_description.name', 
-  'bit_description.desc', 
+  'relationship.id',
+  'event.id',
+  'prefname',
 ],
 
 'account/prefs/prefs.html.tmpl' => [
-  'tab.name', 
-  'tab.description', 
-  'current_tab.name', 
-  'current_tab.description', 
+  'current_tab.label',
+  'current_tab.name',
 ],
 
 );
