@@ -14,11 +14,12 @@
 # Contributor(s): Joel Peshkin <bugreport@peshkin.net>
 #                 Byron Jones <byron@glob.com.au>
 
-# testserver.pl is involked with the baseurl of the Bugzilla installation
+# testserver.pl is invoked with the baseurl of the Bugzilla installation
 # as its only argument.  It attempts to troubleshoot as many installation
 # issues as possible.
 
 use Socket;
+use Bugzilla::Config qw($datadir);
 my $envpath = $ENV{'PATH'};
 use lib ".";
 use strict;
@@ -37,7 +38,7 @@ if ((@ARGV != 1) || ($ARGV[0] !~ /^https?:/))
 
 
 # Try to determine the GID used by the webserver.
-my @pscmds = ('ps -eo comm,gid', 'ps -acxo command,gid');
+my @pscmds = ('ps -eo comm,gid', 'ps -acxo command,gid', 'ps -acxo command,rgid');
 my $sgid = 0;
 if ($^O !~ /MSWin32/i) {
     foreach my $pscmd (@pscmds) {
@@ -82,14 +83,14 @@ to validate webservergroup.\n";
 }
 
 
-# Try to fetch a static file (ant.jpg)
+# Try to fetch a static file (front.png)
 $ARGV[0] =~ s/\/$//;
-my $url = $ARGV[0] . "/ant.jpg";
+my $url = $ARGV[0] . "/skins/standard/index/front.png";
 if (fetch($url)) {
-    print "TEST-OK Got ant picture.\n";
+    print "TEST-OK Got front picture.\n";
 } else {
     print 
-"TEST-FAILED Fetch of ant.jpg failed
+"TEST-FAILED Fetch of skins/standard/index/front.png failed
 Your webserver could not fetch $url.
 Check your webserver configuration and try again.\n";
     exit(1);
@@ -122,6 +123,87 @@ Check your webserver configuration.\n";
     print "TEST-OK Webserver is preventing fetch of $url.\n";
 }
 
+# Test chart generation
+eval 'use GD';
+if ($@ eq '') {
+    undef $/;
+
+    # Ensure major versions of GD and libgd match
+    # Windows's GD module include libgd.dll, guaranteed to match
+    if ($^O !~ /MSWin32/i) {
+        my $gdlib = `gdlib-config --version 2>&1` || "";
+        $gdlib =~ s/\n$//;
+        if (!$gdlib) {
+            print "TEST-WARNING Failed to run gdlib-config; can't compare " .
+                  "GD versions.\n";
+        }
+        else {
+            my $gd = $GD::VERSION;
+    
+            my $verstring = "GD version $gd, libgd version $gdlib";
+    
+            $gdlib =~ s/^([^\.]+)\..*/$1/;
+            $gd =~ s/^([^\.]+)\..*/$1/;
+    
+            if ($gdlib == $gd) {
+                print "TEST-OK $verstring; Major versions match.\n";
+            } else {
+                print "TEST-FAILED $verstring; Major versions do not match.\n";
+            }
+        }
+    }
+
+    # Test GD
+    eval {
+        my $image = new GD::Image(100, 100);
+        my $black = $image->colorAllocate(0, 0, 0);
+        my $white = $image->colorAllocate(255, 255, 255);
+        my $red = $image->colorAllocate(255, 0, 0);
+        my $blue = $image->colorAllocate(0, 0, 255);
+        $image->transparent($white);
+        $image->rectangle(0, 0, 99, 99, $black);
+        $image->arc(50, 50, 95, 75, 0, 360, $blue);
+        $image->fill(50, 50, $red);
+
+        if ($image->can('png')) {
+            create_file("$datadir/testgd-local.png", $image->png);
+            check_image("$datadir/testgd-local.png", 'GD');
+        } else {
+            print "TEST-FAILED GD doesn't support PNG generation.\n";
+        }
+    };
+    if ($@ ne '') {
+        print "TEST-FAILED GD returned: $@\n";
+    }
+
+    # Test Chart
+    eval 'use Chart::Lines';
+    if ($@) {
+        print "TEST-FAILED Chart::Lines is not installed.\n";
+    } else {
+        eval {
+            my $chart = Chart::Lines->new(400, 400);
+
+            $chart->add_pt('foo', 30, 25);
+            $chart->add_pt('bar', 16, 32);
+
+            $chart->png("$datadir/testchart-local.png");
+            check_image("$datadir/testchart-local.png", "Chart");
+        };
+        if ($@ ne '') {
+            print "TEST-FAILED Chart returned: $@\n";
+        }
+    }
+
+    eval 'use Template::Plugin::GD::Image';
+    if ($@) {
+        print "TEST-FAILED Template::Plugin::GD is not installed.\n";
+    }
+    else {
+        print "TEST-OK Template::Plugin::GD is installed.\n";
+    }
+}
+
 sub fetch {
     my $url = shift;
     my $rtn;
@@ -150,7 +232,6 @@ sub fetch {
             select((select(SOCK), $| = 1)[0]);
 
             # get content
-    
             print SOCK "GET $file HTTP/1.0\015\012host: $host:$port\015\012\015\012";
             my $header = '';
             while (defined(my $line = <SOCK>)) {
@@ -169,3 +250,32 @@ sub fetch {
     return($rtn);
 }
 
+sub check_image {
+    my ($local_file, $library) = @_;
+    my $filedata = read_file($local_file);
+    if ($filedata =~ /^\x89\x50\x4E\x47\x0D\x0A\x1A\x0A/) {
+        print "TEST-OK $library library generated a good PNG image.\n";
+        unlink $local_file;
+    } else {
+        print "TEST-WARNING $library library did not generate a good PNG.\n";
+    }
+}
+
+sub create_file {
+    my ($filename, $content) = @_;
+    open(FH, ">$filename")
+        or die "Failed to create $filename: $!\n";
+    binmode FH;
+    print FH $content;
+    close FH;
+}
+
+sub read_file {
+    my ($filename) = @_;
+    open(FH, $filename)
+        or die "Failed to open $filename: $!\n";
+    binmode FH;
+    my $content = <FH>;
+    close FH;
+    return $content;
+}
