@@ -34,6 +34,7 @@ use Bugzilla::Constants;
 use Bugzilla::Search;
 use Bugzilla::User;
 use Bugzilla::BugMail;
+use Bugzilla::Util;
 
 # create some handles that we'll need
 my $template = Bugzilla->template;
@@ -50,7 +51,7 @@ my @seen_schedules = ();
 # allow the database to keep their SQL compiled.
 my $sth_run_queries =
     $dbh->prepare("SELECT " .
-                  "id, query_name, title, onemailperbug " .
+                  "query_name, title, onemailperbug " .
                   "FROM whine_queries " .
                   "WHERE eventid=? " .
                   "ORDER BY sortkey");
@@ -69,12 +70,13 @@ my $sth_next_scheduled_event = $dbh->prepare(
     "LEFT JOIN whine_events " .
     " ON whine_events.id = whine_schedules.eventid " .
     "WHERE run_next <= NOW() " .
-    "ORDER BY run_next LIMIT 1"
+    "ORDER BY run_next " .
+    $dbh->sql_limit(1)
 );
 
 # get all pending schedules matching an eventid
 my $sth_schedules_by_event = $dbh->prepare(
-    "SELECT id, mailto_userid " .
+    "SELECT id, mailto_type, mailto " .
     "FROM whine_schedules " .
     "WHERE eventid=? AND run_next <= NOW()"
 );
@@ -92,15 +94,8 @@ my $sth_schedules_by_event = $dbh->prepare(
 # After that, it looks over each user to see if they have schedules that need
 # running, then runs those and generates the email messages.
 
-# exit quietly if the system is shut down
-if (Param('shutdownhtml')) {
-    exit;
-}
-
-
-# Send whines from the maintainer address. It's not a good idea to use
-# the whine creator address because the admin can make more use of bounces and
-# other replies.
+# Send whines from the address in the 'maintainer' Parameter so that all
+# Bugzilla-originated mail appears to come from a single address.
 my $fromaddress = Param('maintainer');
 
 if ($fromaddress !~ Param('emailregexp')) {
@@ -117,12 +112,16 @@ if (open(NOMAIL, '<', "$datadir/nomail")) {
 }
 
 # get the current date and time from the database
-$sth = $dbh->prepare( 'SELECT DATE_FORMAT( NOW(), "%y,%m,%e,%w,%k,%i")');
+$sth = $dbh->prepare('SELECT ' . $dbh->sql_date_format('NOW()', '%y,%m,%d,%a,%H,%i'));
 $sth->execute;
-my ($now_year, $now_month, $now_day, $now_weekday, $now_hour, $now_minute) =
+my ($now_year, $now_month, $now_day, $now_weekdayname, $now_hour, $now_minute) =
         split(',', $sth->fetchrow_array);
 $sth->finish;
 
+# As DBs have different days numbering, use day name and convert it
+# to the range 0-6
+my $now_weekday = index("SunMonTueWedThuFriSat", $now_weekdayname) / 3;
+
 my @daysinmonth = qw(0 31 28 31 30 31 30 31 31 30 31 30 31);
 # Alter February in case of a leap year.  This simple way to do it only
 # applies if you won't be looking at February of next year, which whining
@@ -166,19 +165,21 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # A time greater than now means it still has to run today
         elsif ($time >= $now_hour) {
             # set it to today + number of hours
-            $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                   "SET run_next=DATE_ADD(CURRENT_DATE(), INTERVAL ? HOUR) " .
-                   "WHERE id=?");
+            $sth = $dbh->prepare("UPDATE whine_schedules " .
+                                 "SET run_next = CURRENT_DATE + " .
+                                 $dbh->sql_interval('?', 'HOUR') .
+                                 " WHERE id = ?");
             $sth->execute($time, $schedule_id);
         }
         # the target time is less than the current time
         else { # set it for the next applicable day
-            my $nextdate = &get_next_date($day);
-            $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                   "SET run_next=" .
-                   "DATE_ADD(?, INTERVAL ? HOUR) " .
-                   "WHERE id=?");
-            $sth->execute($nextdate, $time, $schedule_id);
+            $day = &get_next_date($day);
+            $sth = $dbh->prepare("UPDATE whine_schedules " .
+                                 "SET run_next = (CURRENT_DATE + " .
+                                 $dbh->sql_interval('?', 'DAY') . ") + " .
+                                 $dbh->sql_interval('?', 'HOUR') .
+                                 " WHERE id = ?");
+            $sth->execute($day, $time, $schedule_id);
         }
 
     }
@@ -190,9 +191,11 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # midnight
         my $target_time = ($time =~ /^\d+$/) ? $time : 0;
 
-        $sth = $dbh->prepare( "UPDATE whine_schedules " .
-               "SET run_next=DATE_ADD(?, INTERVAL ? HOUR) " .
-               "WHERE id=?");
+        $sth = $dbh->prepare("UPDATE whine_schedules " .
+                             "SET run_next = (CURRENT_DATE + " .
+                             $dbh->sql_interval('?', 'DAY') . ") + " .
+                             $dbh->sql_interval('?', 'HOUR') .
+                             " WHERE id = ?");
         $sth->execute($target_date, $target_time, $schedule_id);
     }
 }
@@ -220,12 +223,12 @@ sub get_next_event {
     # Loop until there's something to return
     until (scalar keys %{$event}) {
 
-        $dbh->do("LOCK TABLE " .
-                 "whine_schedules WRITE, " .
-                 "whine_events READ, " .
-                 "profiles READ, " .
-                 "groups READ, " .
-                 "user_group_map READ");
+        $dbh->bz_lock_tables('whine_schedules WRITE',
+                             'whine_events READ',
+                             'profiles READ',
+                             'groups READ',
+                             'group_group_map READ',
+                             'user_group_map READ');
 
         # Get the event ID for the first pending schedule
         $sth_next_scheduled_event->execute;
@@ -245,24 +248,51 @@ sub get_next_event {
 
         # Add the users from those schedules to the list
         while (my $row = $sth_schedules_by_event->fetch) {
-            my ($sid, $mailto) = @{$row};
+            my ($sid, $mailto_type, $mailto) = @{$row};
 
             # Only bother doing any work if this user has whine permission
             if ($owner->in_group('bz_canusewhines')) {
-                if (not defined $user_objects{$mailto}) {
-                    if ($mailto == $owner_id) {
-                        $user_objects{$mailto} = $owner;
-                    }
-                    elsif ($whineatothers) {
-                        $user_objects{$mailto} = Bugzilla::User->new($mailto);
+
+                if ($mailto_type == MAILTO_USER) {
+                    if (not defined $user_objects{$mailto}) {
+                        if ($mailto == $owner_id) {
+                            $user_objects{$mailto} = $owner;
+                        }
+                        elsif ($whineatothers) {
+                            $user_objects{$mailto} = Bugzilla::User->new($mailto);
+                        }
                     }
                 }
+                elsif ($mailto_type == MAILTO_GROUP) {
+                    my $sth = $dbh->prepare("SELECT name FROM groups " .
+                                            "WHERE id=?");
+                    $sth->execute($mailto);
+                    my $groupname = $sth->fetch->[0];
+                    my $group_id = Bugzilla::Group::ValidateGroupName(
+                        $groupname, $owner);
+                    if ($group_id) {
+                        my $glist = join(',',
+                            @{Bugzilla::User->flatten_group_membership(
+                            $group_id)});
+                        $sth = $dbh->prepare("SELECT user_id FROM " .
+                                             "user_group_map " .
+                                             "WHERE group_id IN ($glist)");
+                        $sth->execute();
+                        for my $row (@{$sth->fetchall_arrayref}) {
+                            if (not defined $user_objects{$row->[0]}) {
+                                $user_objects{$row->[0]} =
+                                    Bugzilla::User->new($row->[0]);
+                            }
+                        }
+                    }
+                }
+
             }
 
             reset_timer($sid);
         }
 
-        $dbh->do("UNLOCK TABLES");
+        $dbh->bz_unlock_tables();
 
         # Only set $event if the user is allowed to do whining
         if ($owner->in_group('bz_canusewhines')) {
@@ -293,7 +323,7 @@ while (my $event = get_next_event) {
 
     # We loop for each target user because some of the queries will be using
     # subjective pronouns
-    Bugzilla->switch_to_shadow_db();
+    $dbh = Bugzilla->switch_to_shadow_db();
     for my $target (@{$event->{'mailto'}}) {
         my $args = {
             'subject'     => $event->{'subject'},
@@ -318,7 +348,7 @@ while (my $event = get_next_event) {
 
         mail($args);
     }
-    Bugzilla->switch_to_main_db();
+    $dbh = Bugzilla->switch_to_main_db();
 }
 
 ################################################################################
@@ -374,7 +404,7 @@ sub mail {
 
     # now produce a ready-to-mail mime-encoded message
 
-    $args->{'boundary'} = "-----=====-----" . $$ . "--" . time() . "-----";
+    $args->{'boundary'} = "----------" . $$ . "--" . time() . "-----";
 
     $template->process("whine/multipart-mime.txt.tmpl", $args, \$msg)
         or die($template->error());
@@ -395,18 +425,19 @@ sub run_queries {
     my $return_queries = [];
 
     $sth_run_queries->execute($args->{'eventid'});
-    my $queries = {};
+    my @queries = ();
     for (@{$sth_run_queries->fetchall_arrayref}) {
-        $queries->{$_->[0]} = {
-            'name'          => $_->[1],
-            'title'         => $_->[2],
-            'onemailperbug' => $_->[3],
-            'bugs'          => [],
-        };
+        push(@queries,
+            {
+              'name'          => $_->[0],
+              'title'         => $_->[1],
+              'onemailperbug' => $_->[2],
+              'bugs'          => [],
+            }
+        );
     }
 
-    for my $query_id (keys %{$queries}) {
-        my $thisquery = $queries->{$query_id};
+    foreach my $thisquery (@queries) {
         next unless $thisquery->{'name'};   # named query is blank
 
         my $savedquery = get_query($thisquery->{'name'}, $args->{'author'});
@@ -534,6 +565,10 @@ sub reset_timer {
     $sth->execute($schedule_id);
     my ($run_day, $run_time) = $sth->fetchrow_array;
 
+    # It may happen that the run_time field is NULL or blank due to
+    # a bug in editwhines.cgi when this field was initially 0.
+    $run_time ||= 0;
+
     my $run_today = 0;
     my $minute_offset = 0;
 
@@ -565,22 +600,25 @@ sub reset_timer {
 
         my $nextdate = &get_next_date($run_day);
 
-        $sth = $dbh->prepare( "UPDATE whine_schedules " .
-                              "SET run_next=DATE_ADD(?, INTERVAL ? HOUR) " .
-                              "WHERE id=?");
+        $sth = $dbh->prepare("UPDATE whine_schedules " .
+                             "SET run_next = (CURRENT_DATE + " .
+                             $dbh->sql_interval('?', 'DAY') . ") + " .
+                             $dbh->sql_interval('?', 'HOUR') .
+                             " WHERE id = ?");
         $sth->execute($nextdate, $target_time, $schedule_id);
         return;
     }
 
-    # Scheduling is done in terms of whole minutes, so we use DATE_SUB() to
-    # drop the seconds from the time.
     if ($minute_offset > 0) {
+        # Scheduling is done in terms of whole minutes.
+        my $next_run = $dbh->selectrow_array('SELECT NOW() + ' .
+                                             $dbh->sql_interval('?', 'MINUTE'),
+                                             undef, $minute_offset);
+        $next_run = format_time($next_run, "%Y-%m-%d %R");
+
         $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = " .
-                             "DATE_SUB(DATE_ADD(NOW(), INTERVAL ? MINUTE), " .
-                             "INTERVAL SECOND(NOW()) SECOND) " .
-                             "WHERE id=?");
-        $sth->execute($minute_offset, $schedule_id);
+                             "SET run_next = ? WHERE id = ?");
+        $sth->execute($next_run, $schedule_id);
     } else {
         # The minute offset is zero or less, which is not supposed to happen.
         # complain to STDERR
@@ -604,7 +642,7 @@ sub null_schedule {
 # time a schedule should run, excluding today
 #
 # It takes a run_day argument (see check_today, above, for an explanation),
-# and returns an SQL date
+# and returns an integer, representing a number of days.
 sub get_next_date {
     my $day = shift;
 
@@ -657,10 +695,5 @@ sub get_next_date {
             $add_days += $daysinmonth[$now_month];
         }
     }
-
-    # Get a date in whatever format the database will accept
-    $sth = $dbh->prepare("SELECT DATE_ADD(CURRENT_DATE(), INTERVAL ? DAY)");
-    $sth->execute($add_days);
-    return $sth->fetch->[0];
+    return $add_days;
 }
-
