@@ -28,15 +28,24 @@
 # touched for more than the number of days specified in the whinedays param.
 
 use strict;
+use lib '.';
 
 require "globals.pl";
 
 use Bugzilla::BugMail;
+use Bugzilla::Util;
 
-SendSQL("select bug_id,short_desc,login_name from bugs,profiles where " .
-        "(bug_status = 'NEW' or bug_status = 'REOPENED') and " . 
-        "to_days(now()) - to_days(delta_ts) > " . Param('whinedays') .
-        " and userid=assigned_to order by bug_id");
+# Whining is disabled if whinedays is zero
+exit unless Param('whinedays') >= 1;
+
+my $dbh = Bugzilla->dbh;
+SendSQL("SELECT bug_id, short_desc, login_name " .
+        "FROM bugs INNER JOIN profiles ON userid = assigned_to " .
+        "WHERE (bug_status = 'NEW' OR bug_status = 'REOPENED') " .
+        "AND " . $dbh->sql_to_days('NOW()') . " - " .
+                 $dbh->sql_to_days('delta_ts') . " > " .
+                 Param('whinedays') . " " .
+        "ORDER BY bug_id");
 
 my %bugs;
 my %desc;
@@ -63,7 +72,7 @@ foreach my $email (sort (keys %bugs)) {
     my %substs;
     $substs{'email'} = $email . $emailsuffix;
     $substs{'userid'} = $email;
-    my $msg = PerformSubsts($template, \%substs);
+    my $msg = perform_substs($template, \%substs);
 
     foreach my $i (@{$bugs{$email}}) {
         $msg .= "  " . shift(@{$desc{$email}}) . "\n";
